/***************************************************************************
 *   Copyright Brian Ledbetter 2001-2003 <brian@shadowcom.net>             *
 *   Copyright Ravikiran Rajagopal 2003                                    *
 *   ravi@ee.eng.ohio-state.edu                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License (version 2) as   *
 *   published by the Free Software Foundation. (The original KSplash/ML   *
 *   codebase (upto version 0.95.3) is BSD-licensed.)                      *
 *                                                                         *
 ***************************************************************************/

#include <kapplication.h>
#include <kconfig.h>
#include <kdialog.h>
#include <kdebug.h>
#include <kgenericfactory.h>
#include <kglobalsettings.h>
#include <kiconloader.h>
#include <klocale.h>
#include <ksplash/objkstheme.h>
#include <kstandarddirs.h>

#include <qcheckbox.h>
#include <qdesktopwidget.h>
#include <qframe.h>
#include <qhbox.h>
#include <qhbuttongroup.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qvbox.h>
#include <qwidget.h>

#include "thememacclassic.h"
#include "thememacclassic.moc"
#include "icon.h"

/*
 * Embedded images. Using XPM format, since the UIC interface for embedding PNG files
 * is too obscure for my little noggin'. TrollTech guys: How's bout some documentation?
 */
#define LLH_OFF_OFF 0
#define LRH_OFF_OFF 0
#define ULH_OFF_OFF 0
#define URH_OFF_OFF 0
#define LLV_OFF_OFF 0
#define LRV_OFF_OFF 0
#define ULV_OFF_OFF 0
#define URV_OFF_OFF 0
#define LLH_ON_OFF 1
#define LLH_ON_ON 2
#include "preview0.xpm"
#include "preview1.xpm"
#include "preview2.xpm"

K_EXPORT_COMPONENT_FACTORY( libksplashmacclassic, KGenericFactory<ThemeMacClassic> );

MacClassicConfig::MacClassicConfig( QWidget *parent, KConfig *config )
    :ThemeEngineConfig( parent, config )
{

  QPixmap *p;
  p = new QPixmap( preview0_xpm );
  mPreviewImages.append( p );
  p = new QPixmap( preview1_xpm );
  mPreviewImages.append( p );
  p = new QPixmap( preview2_xpm );
  mPreviewImages.append( p );

  QVBox *vbox = new QVBox( this );
  vbox->setSpacing( KDialog::spacingHint() );

  QHButtonGroup *bb = new QHButtonGroup( i18n("Icon Position"), vbox );
  QWidget *widget = new QWidget( bb );

  QGridLayout *grid = new QGridLayout( widget, 9, 9, 1 );
  grid->setRowStretch( 1, 40 );
  grid->setRowStretch( 7, 40 );
  grid->setColStretch( 0, 0 );
  grid->setColStretch( 8, 0 );

  QPushButton *pb;

  // ULH
  pb = new QPushButton( widget );
  pb->setFixedSize( 65, 16 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 0, 1 );

  // LLH
  pb = new QPushButton( widget );
  pb->setFixedSize( 65, 16 );
  pb->setText( "*" );
  mPosButtons.append( pb );
  grid->addWidget( pb, 8, 1 );

  // URH
  pb = new QPushButton( widget );
  pb->setFixedSize( 65, 16 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 0, 7 );

  // LRH
  pb = new QPushButton( widget );
  pb->setFixedSize( 65, 16 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 8, 7 );

  // ULV
  pb = new QPushButton( widget );
  pb->setFixedSize( 16, 65 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 1, 0 );

  // URV
  pb = new QPushButton( widget );
  pb->setFixedSize( 16, 65 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 1, 8 );

  // LLV
  pb = new QPushButton( widget );
  pb->setFixedSize( 16, 65 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 7, 0 );

  // LRV
  pb = new QPushButton( widget );
  pb->setFixedSize( 16, 65 );
  mPosButtons.append( pb );
  grid->addWidget( pb, 7, 8 );

  QHBox *fr = new QHBox( widget );
  fr->setFrameStyle( QFrame::WinPanel );
  fr->setFrameShadow( QFrame::Sunken );
  grid->addMultiCellWidget( fr, 1, 7, 1, 7 );

  mPreview = new QLabel( fr );
  mPreview->setFixedSize( 320, 200 );
  mPreview->setPixmap( *mPreviewImages.at( LLH_OFF_OFF ) );

  //mConfig->setGroup( QString("KSplash Theme: MacClassic") );
  mShowIcons = new QCheckBox( i18n("Show icons as components load."), vbox );
  mShowIcons->setChecked( mConfig->readBoolEntry( "Icons Visible", true ) );

  mIconsJumping = new QCheckBox( i18n("Icons jump while they are starting."), vbox );
  mIconsJumping->setChecked( mConfig->readBoolEntry( "Icons Jumping", true ) );

  connect(mShowIcons,SIGNAL(toggled(bool)),this,SLOT(_slotShowIconsChanged( bool )));
  connect(mIconsJumping,SIGNAL(toggled(bool)),this,SLOT(_slotIconsJumpingChanged( bool )));
}

void MacClassicConfig::_readSettings()
{
}

void MacClassicConfig::_slotShowIconsChanged( bool b )
{
  if( !b )
  {
    mIconsJumping->setDisabled( true );
    mPreview->setPixmap( *mPreviewImages.at( LLH_OFF_OFF ) );

    for( unsigned int i = 0; i < mPosButtons.count(); i++ )
      mPosButtons.at( i )->setDisabled( true );
  }
  else
  {
    for( unsigned int i = 0; i < mPosButtons.count(); i++ )
      mPosButtons.at( i )->setDisabled( false );
    mIconsJumping->setDisabled( false );
    if( mIconsJumping->isChecked() )
      mPreview->setPixmap( *mPreviewImages.at( LLH_ON_ON ) );
    else
      mPreview->setPixmap( *mPreviewImages.at( LLH_ON_OFF ) );
  }
}

void MacClassicConfig::_slotIconsJumpingChanged( bool b )
{
  if( b )
    mPreview->setPixmap( *mPreviewImages.at( LLH_ON_ON ) );
  else
    mPreview->setPixmap( *mPreviewImages.at( LLH_ON_OFF ) );
}

ThemeMacClassic::ThemeMacClassic( QWidget *parent, const char *name, const QStringList &args )
    :ThemeEngine( parent, name, args )
{
  mIcon = mPrevIcon = 0L;
  mIconCount = 0;
  mStdIconWidth = -1;

  _readSettings();
  _initUi();
}

void ThemeMacClassic::_initUi()
{

  _loadSplashScreen();

  setFixedSize( mSplashScreen.width()+15, mSplashScreen.height()+25 );
  setFrameStyle( QFrame::WinPanel );
  setFrameShadow( QFrame::Raised );
  setMargin( 10 );
  setSpacing( 10 );

  QVBox *vbox = new QVBox( this );

  QHBox *frame = new QHBox( vbox );
  frame->setFrameShape( QFrame::WinPanel );
  frame->setFrameShadow( QFrame::Sunken );
  frame->setFixedSize( mSplashScreen.width()+4, mSplashScreen.height()+4 );

  QLabel *lbl = new QLabel( frame );
  lbl->setPixmap( mSplashScreen );
  lbl->setFixedSize( mSplashScreen.width(), mSplashScreen.height() );

  QWidget *foo = new QWidget( vbox );
  foo->setFixedWidth( mSplashScreen.width() );
  foo->setFixedHeight(20);

  QHBox *hbox = new QHBox( vbox );
  QWidget *foo2 = new QWidget( hbox );
  foo2->setFixedSize( mSplashScreen.width()/4, 15);
  mProgressBar = new KProgress( hbox );
  mProgressBar->setFixedSize( mSplashScreen.width()/2, 15 );
  mProgressBar->setTextEnabled( false );
  QWidget *foo3 = new QWidget( hbox );
  foo3->setFixedSize( mSplashScreen.width()/4, 15);

  QWidget *foo4 = new QWidget( vbox );
  foo4->setFixedWidth( mSplashScreen.width() );
  foo4->setFixedHeight(15);

  setFixedWidth( width() );
  setFixedHeight( frame->height() + foo->height() + hbox->height() + foo4->height() + 10 );

  QRect rect = KGlobalSettings::splashScreenDesktopGeometry();

  move( rect.x() + (rect.width() - width())/2,
        rect.y() + (rect.height() - height())/2 );
}

void ThemeMacClassic::_loadSplashScreen()
{
  QPixmap pix;

  // Load THEMEDIR/Splash.png first.
  QString themePixName = locate( "data", mTheme->themeDir() + "/Splash.png" );
  if( !themePixName.isNull() )
    pix = QPixmap( themePixName );

  // If the Theme.rc file (or the user's local preferences) overrides the
  // Splash screen, then use that instead.
  if( !mSplashScreenName.isNull() &&
      (mSplashScreenName != "(Default)") )
    pix = QPixmap( mSplashScreenName );

  // Fallback to the LEGACY file location.
  if( pix.isNull() )
  {
    QString resource_prefix = "pics/";
    if( mTheme->loColor() )
      resource_prefix = "pics/locolor/";
    pix = QPixmap( locate("appdata", resource_prefix + "splash.png") );
  }

  if( pix.isNull() )
    kdDebug() << "ThemeMacClassic::_loadSplashScreen(): Unable to load splash screen. Argh." << endl;

  mSplashScreen = pix;
}

void ThemeMacClassic::slotSetPixmap( const QString& pxn )
{
  QPixmap px = DesktopIcon( pxn );

  // Fallback to a sane icon.
  if( px.isNull() )
    px = DesktopIcon( "go" );

  if( !mIconsVisible )
    return;

  /* (We only use prev_i if jumping is enabled...) */
  if( mIconsJumping && mPrevIcon )
    emit mPrevIcon->slotStopJumping();

  if( mStdIconWidth < 0 )
    mStdIconWidth = DesktopIcon( "go" ).width();

  mIcon = new Icon( ++mIconCount, mStdIconWidth, px, QString::null, mIconPos, mIconsJumping );
  mIcon->show();
  if( mIconsJumping )
  {
    emit mIcon->slotJump();
    mPrevIcon = mIcon;
  }
}

void ThemeMacClassic::_readSettings()
{
  if( !mTheme )
    return;

  KConfig *cfg = mTheme->themeConfig();
  if( !cfg )
    return;

  if( !cfg->hasGroup( QString("KSplash Theme: %1").arg(mTheme->theme()) ) )
    return;
  cfg->setGroup( QString("KSplash Theme: %1").arg(mTheme->theme()) );

  mIconPos = (Icon::Position) cfg->readNumEntry( "Icon Position", 0 );
  mIconsJumping = cfg->readBoolEntry( "Icons Jumping", false );
  mIconsVisible = cfg->readBoolEntry( "Icons Visible", true );
  mSplashScreenName = cfg->readEntry( "Splash Screen", "(Default)" );
}
